@@ -9,7 +9,7 @@ use Carp::Heavy     ();
 use Carp            ();
 use Aspect::Advice  ();
 
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 our @ISA     = 'Aspect::Advice';
 
 sub _install {
@@ -17,6 +17,13 @@ sub _install {
 	my $pointcut = $self->pointcut;
 	my $code     = $self->code;
 
+	# Get the curried version of the pointcut we will use for the
+	# runtime checks instead of the original.
+	# Because $MATCH_RUN is used in boolean conditionals, if there
+	# is nothing to do the compiler will optimise away the code entirely.
+	my $curried   = $pointcut->curry_run;
+	my $MATCH_RUN = $curried ? '$curried->match_run($name, $runtime)' : 1;
+
 	# When an aspect falls out of scope, we don't attempt to remove
 	# the generated hook code, because it might (for reasons potentially
 	# outside our control) have been recursively hooked several times
@@ -30,6 +37,8 @@ sub _install {
 	# Find all pointcuts that are statically matched
 	# wrap the method with advice code and install the wrapper
 	foreach my $name ( $pointcut->match_all ) {
+		my $NAME = $name; # For completeness
+
 		no strict 'refs';
 		my $original = *$name{CODE};
 		unless ( $original ) {
@@ -37,73 +46,72 @@ sub _install {
 		}
 
 		# Any way to set prototypes other than eval?
-		my $prototype = prototype($original);
-		   $prototype = defined($prototype) ? "($prototype)" : '';
+		my $PROTOTYPE = prototype($original);
+		   $PROTOTYPE = defined($PROTOTYPE) ? "($PROTOTYPE)" : '';
 
 		# Generate the new function
 		no warnings 'redefine';
-		eval "sub $name $prototype " . q{{
-			if ( $out_of_scope ) {
+		eval <<"END_PERL"; die $@ if $@;
+		*$NAME = sub $PROTOTYPE {
+			if ( \$out_of_scope ) {
 				# Lexical Aspect is out of scope
-				goto &$original;
+				goto &\$original;
 			}
 
 			# Apply any runtime-specific context checks
-			my $runtime = {};
-			unless ( $pointcut->match_run($name, $runtime) ) {
-				goto &$original;
-			}
+			my \$runtime = {};
+			goto &\$original unless $MATCH_RUN;
 
 			# Prepare the context object
-			my $wantarray = wantarray;
-			my $context   = Aspect::AdviceContext->new(
+			my \$wantarray = wantarray;
+			my \$context   = Aspect::AdviceContext->new(
 				type         => 'before',
-				pointcut     => $pointcut,
-				sub_name     => $name,
-				wantarray    => $wantarray,
-				params       => \@_,
-				return_value => $wantarray ? [ ] : undef,
-				original     => $original,
-				%$runtime,
+				pointcut     => \$pointcut,
+				sub_name     => \$name,
+				wantarray    => \$wantarray,
+				params       => \\\@_,
+				return_value => \$wantarray ? [ ] : undef,
+				original     => \$original,
+				\%\$runtime,
 			);
 
 			# Array context needs some special return handling
-			if ( $wantarray ) {
+			if ( \$wantarray ) {
 				# Run the advice code
-				() = &$code($context);
+				() = &\$code(\$context);
 
-				if ( $context->proceed ) {
-					@_ = $context->params;
-					goto &$original;
+				if ( \$context->proceed ) {
+					\@_ = \$context->params;
+					goto &\$original;
 				}
 
 				# Don't run the original
-				my $rv = $context->return_value;
-				if ( $rv eq 'ARRAY' ) {
-					return @$rv;
+				my \$rv = \$context->return_value;
+				if ( \$rv eq 'ARRAY' ) {
+					return \@\$rv;
 				} else {
-					return ( $rv );
+					return ( \$rv );
 				}
 			}
 
 			# Scalar and void have the same return handling.
 			# Just run the advice code differently.
-			if ( defined $wantarray ) {
-				my $dummy = &$code($context);
+			if ( defined \$wantarray ) {
+				my \$dummy = &\$code(\$context);
 			} else {
-				&$code($context);
+				&\$code(\$context);
 			}
 
 			# Do they want to shortcut?
-			unless ( $context->proceed ) {
-				return $context->return_value;
+			unless ( \$context->proceed ) {
+				return \$context->return_value;
 			}
 
 			# Continue onwards to the original function
-			@_ = $context->params;
-			goto &$original;
-		}};
-		die $@ if $@;
+			\@_ = \$context->params;
+			goto &\$original;
+		};
+END_PERL
 	}
 
 	# Return the lexical descoping hook.
