@@ -4,13 +4,35 @@ use strict;
 use warnings;
 use Aspect::Pointcut ();
 
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 our @ISA     = 'Aspect::Pointcut';
 
 sub new {
 	bless [ $_[1] ], $_[0];
 }
 
+# Logical not inherits it's curryability from the element contained
+# within it. We continue to be needed if and only if something below us
+# continues to be needed as well.
+# For cleanliness (and to avoid accidents) we make a copy of ourself
+# in case our child curries to something other than it's pure self.
+sub curry_run {
+	my $self  = shift;
+	my $child = $self->[0]->curry_run;
+	return unless $child;
+
+	# Handle the special case where the collapsing pointcut results
+	# in a "double not". Fetch the child of our child not and return
+	# it directly.
+	if ( $child->isa('Aspect::Pointcut::NotOp') ) {
+		return $child->[0];
+	}
+
+	# Return our clone with the curried child
+	my $class = ref($self);
+	return $class->new( $child );
+}
+
 sub match_define {
 	return ! shift->[0]->match_define(@_);
 }
